/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2005, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 *
 * Modifications (C) 2006 Russ Price <kxt@fubegra.net>
 * This is loosely based on Asterisk's app_playback.c
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief Play back simulated Panel revertive pulsing sounds based on a four-digit number
 * 
 * \ingroup applications
 */
 
/*
 * $Id: app_rpsim.c,v 1.2 2006/08/24 19:00:51 russ Exp $
 * $Log: app_rpsim.c,v $
 * Revision 1.2  2006/08/24 19:00:51  russ
 * Proper build system for compiling separately from Asterisk
 *
 * Revision 1.1  2006/08/24 18:39:29  russ
 * Initial revision
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/translate.h>
#include <asterisk/utils.h>
#include <asterisk/md5.h>
#include <asterisk/app.h>

static char *tdesc = "Play back simulated Panel revertive pulsing sounds for a four-digit number";

static char *app = "RPsim";

static char *synopsis = "Simulate Panel revertive pulsing";

static char *descrip = 
"  RPsim(number|options):  Mimics the revertive pulsing sounds\n"
"that were a characteristic of calling a Bell System Panel switch\n" 
"through a crossbar tandem. See <http://www.widewebs.com/phonetrips/>\n"
"for some examples. This application uses material from the Evan\n" 
"Doorbell tapes.\n"
"The number should be four digits.\n\n"
"  Options:\n"
"    b_side	- Adds four pulses to second RP digit to simulate\n"
"		  B-side exchange\n"
"    answer	- Answer channel before playing RP digits.\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

void translate(int number, int b_side, int *data);
int playit(struct ast_channel *chan, char *file);

int playit(struct ast_channel *chan, char *file)
{
	int res;
	res = ast_streamfile(chan, file, chan->language);
	if(!res) {
		res = ast_waitstream(chan, "");
		ast_stopstream(chan);
	} else {
		ast_log(LOG_WARNING, "ast_streamfile failed on %s for %s\n", chan->name, file);
		res = 0;
	}
	return res;
}

void translate(int number, int b_side, int *data)
{
	if(number > 9999) {
		ast_log(LOG_WARNING, "RP simulator number should be four digits; truncating %d to %d\n", number, number % 10000);
		number %= 10000;
	}
	int b = (b_side) ? 4 : 0;
	data[0] = (number / 2000) + 1;
	data[1] = (number % 2000) / 500 + b + 1;
	data[2] = (number % 500) / 100 + 1;
	data[3] = (number % 100) / 10 + 1;
	data[4] = (number % 10) + 1;
}

static int rpsim_exec(struct ast_channel *chan, void *data)
{
	int res = 0;
	struct localuser *u;
	int option_b_side = 0;
	int option_answer = 0;
	char *tmp=NULL;
	int translated[5];
	char fpath[80];
	int i, j, k;
	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(number);
		AST_APP_ARG(options);
	);

	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "RPsim requires an argument (number)\n");
		return -1;
	}

	LOCAL_USER_ADD(u);

	tmp = ast_strdupa(data);
	if (!tmp) {
		ast_log(LOG_ERROR, "Out of memory!\n");
		LOCAL_USER_REMOVE(u);
		return -1;	
	}

	AST_STANDARD_APP_ARGS(args, tmp);

	if (args.options) {
		if (strcasestr(args.options, "b_side"))
			option_b_side = 1;
		if (strcasestr(args.options, "answer"))
			option_answer = 1;
	}
	
	translate(atoi(tmp), option_b_side, translated);
	ast_stopstream(chan);
	if (chan->_state != AST_STATE_UP) {
		if(option_answer)
			res = ast_answer(chan);
		if (!res)
			res = ast_safe_sleep(chan, 500);
	}
	res = playit(chan, "rpsim/begin");
	for(i=0; i<5; i++) {
		sprintf(fpath, "rpsim/start%d", i);
		res = playit(chan, fpath);

		sprintf(fpath, "rpsim/first%d", i);
		if(translated[i] > 1)
			res = playit(chan, fpath);

		sprintf(fpath, "rpsim/middle%d", i);
		k = translated[i] - 2;
		for(j=0; j<k; j++)
			res = playit(chan, fpath);

		sprintf(fpath, "rpsim/last%d", i);
		res = playit(chan, fpath);
	}
	res = playit(chan, "rpsim/end");
	LOCAL_USER_REMOVE(u);
	return res;
}

int unload_module(void)
{
	int res;

	res = ast_unregister_application(app);
	
	STANDARD_HANGUP_LOCALUSERS;

	return res;	
}

int load_module(void)
{
	return ast_register_application(app, rpsim_exec, synopsis, descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
